/*
	Copyright (c) 2003 Michael Carrato. All rights reserved.	

	Permission to use, modify, copy, and distribute this software 
	and its documentation is hereby granted without fee, 
	provided that the above copyright notice appears in all copies.
	This software is provided "as is" without express or implied 
    warranty.
*/

//disable annoying MSC warning about debug symbols
#ifdef _MSC_VER
#pragma warning (disable:4786)
#endif // _MSC_VER

#include "CompactSet.h"

#include <iostream>

//typedef CompactColl<int,std::less<int>,MinimizeSpacePolicy> SESType;
//typedef CompactColl<int> SESType;
void testSESet()
{
    typedef CompactSet<int,std::less<int>,MaximizeSpeedPolicy> SESType;
    typedef std::set<int> ControlSet;
//    typedef CompactSet<int,std::less<int>,MinimizeSpacePolicy> SESType;
    std::set<int> control;
    SESType sa;
    for(int i=0;i<50000;++i)
    {
		if(i % 1000 == 0)
		{
			std::cout << i << "              \r" << std::flush;
		}
        unsigned newRand = (unsigned) (rand() * rand()) % 100000;
        unsigned action = (rand() % 3001)/1000;  
        switch(action)
        {
        case 0:
            {
                std::pair<ControlSet::iterator,bool> cRet = control.insert(newRand);
                std::pair<SESType::iterator,bool> csRet = sa.insert(newRand);
                assert(cRet.second == csRet.second);
                assert(*cRet.first == *csRet.first);
                --(++csRet.first);
                assert(*cRet.first == *csRet.first);
            }
            break;
        case 1:
            {
                ControlSet::size_type cRet = control.erase(newRand);
                SESType::size_type csRet = sa.erase(newRand);
                assert(cRet == csRet);
            }
            break;
        case 2:
            {
                ControlSet::iterator cRet = control.find(newRand);
                SESType::iterator csRet = sa.find(newRand);
                if(cRet == control.end())
                {
                    assert(csRet == sa.end());
                }
                else
                {
                    assert(*csRet == *cRet);
                    --(++csRet);
                    assert(*csRet == *cRet);
                }
            }
            break;
        //this action is much more computation intensive, and is less frequent.
        case 3:
            {
                std::vector<int> vec1,vec2,vec3;
		std::copy(sa.begin(),sa.end(),std::back_inserter(vec1));
		std::copy(sa.rbegin(),sa.rend(),std::back_inserter(vec3));
		std::reverse(vec3.begin(),vec3.end());
		std::copy(control.begin(),control.end(),std::back_inserter(vec2));
		assert(vec1 == vec2);
		assert(vec3 == vec2);
            }
            break;
        }
/*        std::copy(control.begin(),control.end(), std::ostream_iterator<unsigned>(std::cout,","));
        std::cout << "\n";
        std::copy(sa.begin(),sa.end(), std::ostream_iterator<int>(std::cout,","));
        std::cout << "\n\n";*/
    }
    sa.compact();
    std::cout << "total elements: test=" << sa.size() 
            << ", control=" << control.size() << "\n";
    std::set<int>::const_iterator ctrlIt, ctrlItEnd = control.end();
    SESType::const_iterator saIt, saItEnd = sa.end();
    for(ctrlIt = control.begin(), saIt = sa.begin(); 
            ctrlIt != ctrlItEnd && saIt != saItEnd;
            ++ ctrlIt, ++saIt)
    {
        if(*ctrlIt != *saIt)
            break;
    }
    if(ctrlIt == ctrlItEnd && saIt == saItEnd)
    {
        std::cout << "test identical to control\n";
    }
    else
    {
        std::cout << "ERROR: test different from control!\n";
    }
//    char c;
//    std::cin >> c;
}
