/*
	Copyright (c) 2003 Michael Carrato. All rights reserved.	

	Permission to use, modify, copy, and distribute this software 
	and its documentation is hereby granted without fee, 
	provided that the above copyright notice appears in all copies.
	This software is provided "as is" without express or implied 
    warranty.
*/

#if !defined(AFX_COMPACTSET_H__2FF1D321_9C17_4F6D_A8D4_E1CA4628DB71__INCLUDED_)
#define AFX_COMPACTSET_H__2FF1D321_9C17_4F6D_A8D4_E1CA4628DB71__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "CompactColl.h"
#include <set>

//compact set collection traits
template <class T, class P,class POL, class A>
struct CSetCT
{
    typedef T key_type;
    typedef T value_type;
    typedef T AssignableValueType;
    typedef P key_compare;
    typedef P value_compare;
    typedef A allocator_type;
    typedef typename allocator_type::size_type size_type;
    typedef POL PolicyTagType;

	typedef allocator_type AssignableAllocatorType;
    typedef std::set<T,P,A> DynSetType;

    key_compare kc;
    allocator_type al;
    CSetCT(const key_compare& pred, 
        const allocator_type& alloc)
        : kc(pred), al(alloc)
    { }
    CSetCT()
    { }

    bool less(const key_type& lhs, const key_type& rhs) const
    {
        return kc(lhs,rhs);
    }
    void assign(value_type& lhs, const value_type& rhs) const
    {
        lhs = rhs;
    }
    const key_type& extractKey(const value_type& val) const
    {
        return val;
    }
    key_type& extractKey(value_type& val) const
    {
        return val;
    }
    const key_compare& key_comp() const
    {
        return kc;
    }
    const value_compare& value_comp() const
    {
        return kc;
    }
    const allocator_type& get_allocator() const
    {
        return al;
    }
};


template<class T,class P = std::less<T>,class POLICY = MaximizeSpeedPolicy,class A = std::allocator<T> >
class CompactSet
: public CompactColl<CSetCT<T,P,POLICY,A> >
{
    typedef CompactColl<CSetCT<T,P,POLICY,A> > ImplType;
public:
    typedef T key_type;
    typedef T value_type;
    typedef P key_compare;
    typedef P value_compare;
    typedef A allocator_type;
    typedef typename allocator_type::size_type size_type;
    typedef typename allocator_type::difference_type difference_type;

    typedef typename ImplType::iterator iterator;
    typedef typename ImplType::const_iterator const_iterator;    
    typedef typename ImplType::reverse_iterator reverse_iterator;
    typedef typename ImplType::const_reverse_iterator const_reverse_iterator;

    explicit CompactSet(const key_compare& pred = key_compare(), 
        const allocator_type& alloc = allocator_type())
        : ImplType(CSetCT<T,P,POLICY,A>(pred,alloc))
    {
    }
};

template<class T,class P,class POLICY,class A>
inline bool operator==(const CompactSet<T,P,POLICY,A>& lhs, const CompactSet<T,P,POLICY,A>& rhs)
{
    return (lhs.size() == rhs.size() 
        && std::equal(lhs.begin(),lhs.end(),rhs.begin()));
}

template<class T,class P,class POLICY,class A>
inline bool operator!=(const CompactSet<T,P,POLICY,A>& lhs, const CompactSet<T,P,POLICY,A>& rhs)
{
    return ! (lhs == rhs);
}

template<class T,class P,class POLICY,class A>
inline bool operator<(const CompactSet<T,P,POLICY,A>& lhs, const CompactSet<T,P,POLICY,A>& rhs)
{
    return std::lexicographical_compare(lhs.begin(),lhs.end(),rhs.begin(),rhs.end());
}

template<class T,class P,class POLICY,class A>
inline bool operator>(const CompactSet<T,P,POLICY,A>& lhs, const CompactSet<T,P,POLICY,A>& rhs)
{
    return rhs < lhs;
}

template<class T,class P,class POLICY,class A>
inline bool operator<=(const CompactSet<T,P,POLICY,A>& lhs, const CompactSet<T,P,POLICY,A>& rhs)
{
    return ! (rhs < lhs);
}

template<class T,class P,class POLICY,class A>
inline bool operator>=(const CompactSet<T,P,POLICY,A>& lhs, const CompactSet<T,P,POLICY,A>& rhs)
{
    return ! (lhs < rhs);
}

#ifndef _MSC_VER
namespace std
{

template<class T,class P,class POLICY,class A>
inline void swap(::CompactSet<T,P,POLICY,A>& lhs, ::CompactSet<T,P,POLICY,A>& rhs)
{
    std::cout << "std::swap\n";
    lhs.swap(rhs);
}
}
#endif

#endif // !defined(AFX_COMPACTSET_H__2FF1D321_9C17_4F6D_A8D4_E1CA4628DB71__INCLUDED_)
