/*
	Copyright (c) 2003 Michael Carrato. All rights reserved.	

	Permission to use, modify, copy, and distribute this software 
	and its documentation is hereby granted without fee, 
	provided that the above copyright notice appears in all copies.
	This software is provided "as is" without express or implied 
    warranty.
*/

#if !defined(AFX_ITERADAPTER_H__2F47FAA0_633A_47D4_8313_5D58C18BCAEB__INCLUDED_)
#define AFX_ITERADAPTER_H__2F47FAA0_633A_47D4_8313_5D58C18BCAEB__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#pragma warning (disable:4786)
#pragma warning (disable:4284)
#endif // _MSC_VER > 1000


template <class ITERIMPL>
class CIterAdapter
{
public:
    typedef typename ITERIMPL::iterator_category iterator_category;
    typedef typename ITERIMPL::value_type value_type;
    typedef value_type* pointer;
    typedef value_type& reference;
    typedef const value_type* const_pointer;
    typedef const value_type& const_reference;
    typedef typename ITERIMPL::size_type size_type;
    typedef typename ITERIMPL::difference_type difference_type;
    CIterAdapter(const ITERIMPL& i)
        : impl_(i)
    { }
    CIterAdapter()
    { }
    const_reference operator*() const
    {
        return impl_.dereference();
    }
    const_pointer operator->() const
    {
        return (&**this); 
    }
    CIterAdapter& operator++()
    {
        impl_.increment();
        return *this;
    }
    CIterAdapter operator++(int)
    {
        CIterAdapter ret = *this;
	++*this;
        return ret;
    }
    CIterAdapter& operator--()
    {
        impl_.decrement();
        return *this; 
    }
    CIterAdapter operator--(int)
    {
        CIterAdapter ret = *this;
        --*this;
        return ret; 
    }
    bool operator==(const CIterAdapter& rhs) const
    {
        return impl_.equals(rhs.impl_);
    }
    bool operator!=(const CIterAdapter& rhs) const
    {
        return (!(*this == rhs)); 
    }
	//need to have this public accessor for some containers. A friendship would be better.
	ITERIMPL& getImpl_()
	{
		return impl_;
	}
protected:
    ITERIMPL impl_;
};

template <class ITERIMPL>
class IterAdapter : public CIterAdapter<ITERIMPL>
{
public:
    typedef typename ITERIMPL::value_type value_type;
    typedef value_type* pointer;
    typedef value_type& reference;

    IterAdapter(const ITERIMPL& i)
    { 
        impl_ = i;
    }
    IterAdapter()
    { }
    reference operator*() const
    {
        return impl_.dereference();
    }
    pointer operator->() const
    {
        return (&**this); 
    }
    IterAdapter& operator++()
    {
        impl_.increment();
        return *this;
    }
    IterAdapter operator++(int)
    {
        IterAdapter ret = *this;
	++*this;
        return ret;
    }
    IterAdapter& operator--()
    {
        impl_.decrement();
        return *this; 
    }
    IterAdapter operator--(int)
    {
        IterAdapter ret = *this;
        --*this;
        return ret; 
    }
};

#endif // !defined(AFX_ITERADAPTER_H__2F47FAA0_633A_47D4_8313_5D58C18BCAEB__INCLUDED_)
