/*
	Copyright (c) 2003 Michael Carrato. All rights reserved.	

	Permission to use, modify, copy, and distribute this software 
	and its documentation is hereby granted without fee, 
	provided that the above copyright notice appears in all copies.
	This software is provided "as is" without express or implied 
    warranty.
*/

#if !defined(AFX_TIMEOFDAY_H__DBA78EB9_4A80_4718_AE19_64A999BDE4A6__INCLUDED_)
#define AFX_TIMEOFDAY_H__DBA78EB9_4A80_4718_AE19_64A999BDE4A6__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <assert.h>
class TimeOfDay;
#ifdef _MSC_VER
#define _TOD_USE_WINDOWS_TIME
#endif
#ifdef __BORLANDC__
#define _TOD_USE_WINDOWS_TIME
#endif

#ifdef _TOD_USE_WINDOWS_TIME
//define an equivalent FILETIME structure without including <windows.h>
//since including <windows.h> in a header would likely cause
//hysteria.
//<windows.h> is included in TimeOfDay.cc
typedef unsigned __int64 UInt64Type;
struct TimeOfDayImpl__Win32 //this
{
    unsigned __int64 fileTime_;
    TimeOfDayImpl__Win32(__int64 ft = 0)
        :fileTime_(ft)
    {
    }
    TimeOfDayImpl__Win32 minus(const TimeOfDayImpl__Win32& rhs) const
    {
        if(fileTime_ < rhs.fileTime_)
            return TimeOfDayImpl__Win32();
        return fileTime_ - rhs.fileTime_;
    }
    UInt64Type totalSeconds() const
    {
        return (unsigned long)(fileTime_ / 10000000);
    }
    UInt64Type totalMilliSeconds() const
    {
        return (unsigned long)(fileTime_ / 10000);
    }
    UInt64Type totalMicroSeconds() const
    {
        return (unsigned long)(fileTime_ / 10);
    }
    void setToCurrentTime();
};
typedef TimeOfDayImpl__Win32 TimeOfDayImpl;
#else
#ifdef __GNUC__
#include <sys/time.h>
typedef unsigned long long UInt64Type;
struct TimeOfDayImpl__GCC
{
    struct timeval tv;
    UInt64Type getAbsoluteMicroSecs() const
    {
        UInt64Type ret = tv.tv_sec;
        ret *= 1000000;
	ret += tv.tv_usec;
	return ret;
    }
    TimeOfDayImpl__GCC(UInt64Type usecs=0)
    {
        tv.tv_sec = usecs / 1000000;
        tv.tv_usec = usecs % 1000000;
    }
    TimeOfDayImpl__GCC minus(const TimeOfDayImpl__GCC& rhs) const
    {
        if(getAbsoluteMicroSecs() < rhs.getAbsoluteMicroSecs())
            return TimeOfDayImpl__GCC();
        return getAbsoluteMicroSecs() - rhs.getAbsoluteMicroSecs();
    }
    UInt64Type totalSeconds() const
    {
        return (unsigned long)(getAbsoluteMicroSecs() / 1000000);
    }
    UInt64Type totalMilliSeconds() const
    {
        return (unsigned long)(getAbsoluteMicroSecs() / 1000);
    }
    UInt64Type totalMicroSeconds() const
    {
        return (unsigned long)(getAbsoluteMicroSecs());
    }
    void setToCurrentTime()
    {
        gettimeofday(&tv,0);
    }
};
typedef TimeOfDayImpl__GCC TimeOfDayImpl;
#endif //#ifdef __GNUC__
#endif //#ifdef _TOD_USE_WINDOWS_TIME ... #else

class TimeOfDay  
{
    TimeOfDayImpl impl_;
    TimeOfDay(const TimeOfDayImpl& impl)
        :impl_(impl)
    {
    }
public:
    static void DoTests();
    TimeOfDay()
    {
    }
    TimeOfDay operator-(const TimeOfDay& rhs) const
    {
        return TimeOfDay(impl_.minus(rhs.impl_));
    }
    UInt64Type totalSeconds() const
    {
        return impl_.totalSeconds();
    }
    UInt64Type totalMilliSeconds() const
    {
        return impl_.totalMilliSeconds();
    }
    UInt64Type totalMicroSeconds() const
    {
        return impl_.totalMicroSeconds();
    }
    void setToCurrentTime()
    {
        impl_.setToCurrentTime();
    }
};

#endif // !defined(AFX_TIMEOFDAY_H__DBA78EB9_4A80_4718_AE19_64A999BDE4A6__INCLUDED_)
