#
# findlintlibs.sh - search for lint libraries
#
# $Id: findlintlibs.sh,v 1.1.1.1 2003/05/18 22:48:30 jonm Exp $

#
# Find which libraries have lint counterparts
#
for lib in "${LIBS[@]}"
do
	case "$lib" in
	  -l*)
		# Remap library name to lint name
		lintlib="${LINTLIBPFX}${lib#-l}.${LINTLIBEXT}"
		
		# Look for the lint library
		for lp in "${STDLPATHS[@]}" "${LPATHS[@]}"
		do
			lintlibfullpath="${lp#-L}/$lintlib"
			if test -f "$lintlibfullpath"
			then
				LINTLIBS[${#LINTLIBS[@]}]="$lintlibfullpath"
				break; # out of Look for the lint library
			fi
		done
		;;

	  *) # Library with an absolute or relative path to the real filename
	    case "$lib" in
			*lib*.a)
				libdir="$(dirname $lib)/"
				test $libdir = "./" && libdir=""
				libbase="$(basename $lib .a)"
				lintlibfullpath="${libdir}${LINTLIBPFX}${libbase#lib}.${LINTLIBEXT}"
				;;
				
			*lib*.so*)
				libdir="$(dirname $lib)/"
				test $libdir = "./" && libdir=""
				libbase="$(basename ${lib%.so*})"				
				lintlibfullpath="${libdir}${LINTLIBPFX}${libbase#lib}-so.${LINTLIBEXT}"
				#
				# Check for -lo version in case lint library built
				# using the lint wrapper/libtool.  If not present,
				# fall back to the normal lint library name.
				#
				if test ! -f "$lintlibfullpath"
				then
					lintlibfullpath="${libdir}${LINTLIBPFX}${libbase#lib}.${LINTLIBEXT}"
				fi
				;;
				
			*.a) # Archive library not starting with lib
				libdir="$(dirname $lib)/"
				test $libdir = "./" && libdir=""
				libbase="$(basename $lib .a)"
				lintlibfullpath="${libdir}${LINTLIBPFX}${libbase}.${LINTLIBEXT}"
				;;
				
			*.so*) # Shared object not starting with lib
				libdir="$(dirname $lib)/"
				test $libdir = "./" && libdir=""
				libbase="$(basename ${lib%.so*})"
				#
				# Check for -lo version in case lint library built
				# using the lint wrapper/libtool.  If not present,
				# fall back to the normal lint library name.
				#
				lintlibfullpath="{$libdir}${libbase}-so.${LINTLIBEXT}"
				if test ! -f "$lintlibfullpath"
				then
					lintlibfullpath="${libdir}${libbase}.${LINTLIBEXT}"
				fi
				;;
		esac
		
		if test -f "$lintlibfullpath"
		then
			LINTLIBS[${#LINTLIBS[@]}]="$lintlibfullpath"
		fi
		;;
	esac
done
