#
# Stage 1 lint compile for lints supporting -c
#


#
# Save any SRCS.ln files if there is a different target and put back
# after linting.  Some lints do not support choosing the output filename
# for first stage parsing.
#
if test -n "${TARGET}"
then
	if ((${#SRCS[*]} > 1))
	then
		echo "Lint wrapper only supports single source file with -o (${#SRCS[*]} given)" >&2
		exit $REAL_RET
	fi

	# Remap the target object file name
	LNTARGET="${TARGET}"
	for xform in "${OBJXFORMS[@]}"
	do
		LNTARGET=$(echo "${LNTARGET}" | sed -e "$xform")
	done
	
fi

#
# Check for workaround for libtool.  Some C compilers do not
# support -c and -o when emitting the .lo files that libtool uses.
# Libtool does a mv on the file after the C compiler has finished
# so lint objects will be created under the wrong name.  If
# TARGETLO is set - but not target name given, append -lo to the lint
# target name.
if test "$TARGETLO" = "Y" -a -z "$TARGET"
then
	#
	# If no target already selected, build it from the source file.
	#
	if ((${#SRCS[@]} > 1))
	then
		echo "Lint wrapper only supports single source file with -o" >&2
		exit $REAL_RET
	fi

	# Convert from some/path/src.c to src-lo.ln
	LNTARGET="${SRCS[0]%.*}-lo.${LINTOBJEXT}"
	LNTARGET="${LNTARGET##*/}"
fi
#
# Check if an existing .ln file will be stamped over, if so rename
# temporarily.
#
LNOUTPUT="${SRCS[0]%.*}.${LINTOBJEXT}"
LNOUTPUT="${LNOUTPUT##*/}"

if test -n "$LNTARGET" -a -f "$LNOUTPUT"
then
	LNTARGETTMP="${SRCS[0]%.*}.${LINTOBJEXT}.$$"
	LNTARGETTMP="${LNTARGETTMP##*/}"
	
	mv "${LNOUTPUT}" "${LNTARGETTMP}"
fi

echo
echo "Lint compiling ${SRCS[*]}"
echo

test -n "$LINTDBG" && echo Running: ${LINT} ${LINTCFLAGS} ${INCS[@]} ${DEFS[@]} ${LINTCOMPILEFLAGS} ${SRCS[@]}

${LINT} ${LINTCFLAGS} "${INCS[@]}" "${DEFS[@]}" ${LINTCOMPILEFLAGS} "${SRCS[@]}"

#
# Check if the output .ln file needs to be renamed - if a .lo file
# is being targeted.
#
if test -n "${LNTARGET}"
then

	if test "${LNTARGET}" != "${LNOUTPUT}"
	then
		mv "${LNOUTPUT}" "${LNTARGET}"
	fi
	
	# Put back the original .ln file if it exists
	test -n "${LNTARGETTMP}" && mv "${LNTARGETTMP}" "${LNOUTPUT}"
fi



