#
# lintstatlib.sh
#
# $Id: lintstatlib.sh,v 1.1.1.1 2003/05/18 22:48:30 jonm Exp $
#
# Create a lint library by interposing AR.  Need to support creation and
# extraction to work with libtool.
#
# When creating/updating a library maintain a parallel archive called
# libname.aln with all of the .ln files.  After the update, extract
# all of the .ln files to a temporary directory and run lint to create
# llib-lname.ln
#
# When extracting just extract the .ln files to the current directory
# as requested.
#
# REALAR must be set to point to the real ar program to use.
#


#
# Remap the target name to be the lint version of it
# LNLIBNAME is the stub to be used for the lint library name
# ALNTARGET is the name of the .ln archive
#
LINTTARGET="${TARGET%.*}"
LINTTARGET="${LINTTARGET##*/}"
LINTTARGET="${LINTTARGET#lib}"

#
# Remap the target name
#
ALNTARGET="${TARGET}"
for xform in "${ALNXFORMS[@]}"
do
	ALNTARGET=$(echo "${ALNTARGET}" | sed -e "$xform")
done

if test "$ARUPDATE" = "Y"
then
	SRCDIR="$PWD"
	ALNDIR="lintar.$$"	

	#
	# If updating the library need to the new .ln files with the
	# one already in the archive to build the lint library
	#	
	if test -f "${SRCDIR}/${ALNTARGET}"
	then
		#
		# Create a temporary directory to update the lint library
		#

		test -f "$ALNDIR" -o -d "$ALNDIR" && rm -rf "$ALNDIR" 
		mkdir "$ALNDIR"

		#
		# Extract the current contents of the library into the temp dir
		#

		cd "$ALNDIR"
		${REALAR} x "${SRCDIR}/${ALNTARGET}"
		cd ..

		# Copy all of the object files into the temporary lint directory
		# making sure to do it after the extract in case any
		# are being updated.
		cp "${OBJS[@]}" "$ALNDIR"

		# Rebuild the lint library from the .ln files in the temp dir and
		# copy back the lint library if successful.
		cd "$ALNDIR"
		echo
		echo "Updating lint library ${LINTTARGET} from " *.${LINTOBJEXT}
		echo "There may be duplicate messages if the library is being appended to."
		echo
		test -n "$LINTDBG" && echo Running ${LINT} ${LINTLIBFLAGS} "${LINTLIBS[@]}" ${LINTLIBTGT}${LINTTARGET} *.${LINTOBJEXT}
		${LINT} ${LINTLIBFLAGS} "${LINTLIBS[@]}" ${LINTLIBTGT}${LINTTARGET} *.${LINTOBJEXT} && cp ${LINTLIBPFX}${LINTTARGET}.${LINTLIBEXT} ..
		cd ..
		rm -rf "$ALNDIR"
	else
		echo
		echo "Creating lint library ${LINTTARGET} from " "${OBJS[@]}"
		echo
		test -n "$LINTDBG" && echo Running ${LINT} ${LINTLIBFLAGS} "${LINTLIBS[@]}" ${LINTLIBTGT}${LINTTARGET} "${OBJS[@]}"
		${LINT} ${LINTLIBFLAGS} "${LINTLIBS[@]}" ${LINTLIBTGT}${LINTTARGET} "${OBJS[@]}"
	fi
	
	
	#
	# Update the .lna archive in parallel with the real archive
	#
	${REALAR} $ARMODE "$ALNTARGET" "${OBJS[@]}"
	
elif test "$AREXTRACT" = "Y"
then
	${REALAR} $ARMODE "${ALNTARGET}" "${OBJS[@]}"
fi
